/*
   BILLGPS.H - include file for VxD BILLGPS

   Free software Copyright (c) 1996 Lew Perin

*/

#ifndef __BILLGPS_H__
#define __BILLGPS_H__

#define USE_NDIS
#include <vtoolsc.h>

/*
  Now some constants from WINIOCTL.H:
*/

#define FILE_DEVICE_UNKNOWN             0x00000022
#define METHOD_BUFFERED                 0
#define METHOD_NEITHER                  3
#define FILE_ANY_ACCESS                 0
#define CTL_CODE( DeviceType, Function, Method, Access ) (  \
((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) \
)

#include "..\shared.h"

#define HOOK_DEBUG 1             /* Trace our hooking of NdisSend()?  */

#define BILLGPS_Major 1
#define BILLGPS_Minor 27
#define BILLGPS_DeviceID UNDEFINED_DEVICE_ID

#define NDIS_Init_Order         0x02000 + VNETBIOS_INIT_ORDER
#define MAC_Init_Order          0x02000 + NDIS_Init_Order
#define PROTOCOL_Init_Order     0x02000 + MAC_Init_Order

#define BILLGPS_Init_Order PROTOCOL_Init_Order

#define BILLGPS_NDIS_MAJOR_VERSION	0x03
#define BILLGPS_NDIS_MINOR_VERSION	0x0A

#define TRANSPORT_NAME "BILLGPS"

#define MAX_OPEN 4
#define MAX_REQUESTS   4

typedef struct _BgpsReserved   {
  LIST_ENTRY listElement;
  char*	lpBuffer;
  DWORD	cbBuffer;
  DWORD* lpcbBytesReturned;
  OVERLAPPED* lpoOverlapped;
  DWORD	hDevice;
  DWORD	tagProcess;
}  
BgpsReserved, *PBgpsReserved;

#define RESERVED(_p) ((PBgpsReserved)((_p)->ProtocolReserved))

typedef struct _INTERNAL_REQUEST 
{
  BgpsReserved Reserved;
  NDIS_REQUEST Request;
} 
INTERNAL_REQUEST, *PINTERNAL_REQUEST;


//
// Port device extension.
//

enum { AdapterNameMax = 6 };

typedef struct _OpenInstance 
{
  LIST_ENTRY ListElement;

  DWORD	tagProcess;
  DWORD hDevice;
  NDIS_HANDLE adapterHandle;    // returned from NdisOpenAdapter
  NDIS_HANDLE bindAdapterContext; // needed by NdisCompleteBindAdapter
  DWORD bindEvent;              // to notify app of bind completion
  NDIS_HANDLE PacketPool;
  NDIS_HANDLE BufferPool;

  NDIS_SPIN_LOCK RcvQSpinLock;
  LIST_ENTRY RcvList;

#if 0                           // These VPACKET things we don't need:

  NDIS_SPIN_LOCK RequestSpinLock;
  LIST_ENTRY RequestList;

  NDIS_SPIN_LOCK ResetSpinLock;
  LIST_ENTRY ResetIrpList;

  INTERNAL_REQUEST Requests[MAX_REQUESTS];

#endif

  char adapterName[AdapterNameMax];       // something like 000x
  UCHAR macAddress[MacAddressSize];
  union {
    NDIS_MEDIUM medium;
    UINT selectedMediumIndex;
  } u1;
  NDIS_REQUEST request;         // NDIS_REQUEST; can't be on stack!
  NDIS_REQUEST filterBlock;
  UINT packetFilter;
  BOOL promiscuous;             // are we accepting any old packet?
  /* Now come statuses that we keep around so we can dump 'em: */
  NDIS_STATUS openErrorStatus;
  NDIS_STATUS openAdapterCompleteStatus;
  NDIS_STATUS macAddressReqStatus;
  NDIS_STATUS setFilterStatus;
  NDIS_STATUS transferDataStatus;
  NDIS_STATUS unbindStatus;
} 
OpenInstance, *POpenInstance;

#define  TRANSMIT_PACKETS    16

/*
   NDIS 3.1 PROTOCOL FUNCTION PROTOTYPES
*/

VOID NDIS_API
BgpsOpenAdapterCompleteHandler(IN NDIS_HANDLE abindingContext,
                               IN NDIS_STATUS astatus,
                               IN NDIS_STATUS aopenErrorStatus);

VOID NDIS_API
BgpsCloseAdapterCompleteHandler(IN NDIS_HANDLE abindingContext,
                                IN NDIS_STATUS astatus);

VOID NDIS_API
BgpsSendCompleteHandler(IN NDIS_HANDLE abindingContext,
                        IN PNDIS_PACKET apacket,
                        IN NDIS_STATUS astatus);

VOID NDIS_API
BgpsTransferDataCompleteHandler(IN NDIS_HANDLE abindingContext,
                                IN PNDIS_PACKET apacket,
                                IN NDIS_STATUS astatus,
                                IN UINT abytesTransferred);

VOID NDIS_API
BgpsResetCompleteHandler(IN NDIS_HANDLE abindingContext,
                         IN NDIS_STATUS astatus);

VOID NDIS_API
BgpsRequestCompleteHandler(IN NDIS_HANDLE abindingContext,
                           IN PNDIS_REQUEST andisRequest,
                           IN NDIS_STATUS astatus);

NDIS_STATUS NDIS_API
BgpsReceiveHandler(IN NDIS_HANDLE abindingContext,
                   IN NDIS_HANDLE amacReceiveContext,
                   IN PVOID aheaderBuffer,
                   IN UINT aheaderBufferSize,
                   IN PVOID alookAheadBuffer,
                   IN UINT alookaheadBufferSize,
                   IN UINT apacketSize);

VOID NDIS_API
BgpsReceiveCompleteHandler(IN NDIS_HANDLE NdisBindingContext);

VOID NDIS_API
BgpsStatusHandler(IN NDIS_HANDLE abindingContext,
                  IN NDIS_STATUS ageneralStatus,
                  IN PVOID astatusBuffer,
                  IN UINT astatusBufferSize);

VOID NDIS_API
BgpsStatusCompleteHandler(IN NDIS_HANDLE abindingContext);


/*
   BIND_ADAPTER_HANDLER, UNBIND_ADAPTER_HANDLER, UNLOAD_PROTOCOL
   are valid for NDIS 3.1 protocols
*/

VOID NDIS_API
BgpsBindAdapterHandler(OUT PNDIS_STATUS astatusP,
                       IN NDIS_HANDLE abindAdapterContext,
                       IN PNDIS_STRING anadapterName,
                       IN PVOID asystemSpecific1,
                       IN PVOID asystemSpecific2);

VOID NDIS_API
BgpsUnbindAdapterHandler(OUT PNDIS_STATUS astatusP,
                         IN NDIS_HANDLE aprotocolBindingContext,
                         IN NDIS_HANDLE anunbindAdapterContext);

VOID NDIS_API BgpsUnloadProtocolHandler(void);

/*
  Non-NDIS Prototypes
*/

NDIS_STATUS
BgpsAllocatePacketBuffer(POpenInstance pOpen,
                         PNDIS_PACKET *pPacket, // 17.2.97
                         PIOCTLPARAMS aPIP);

DWORD BgpsRead(PIOCTLPARAMS aPIP);

void BgpsDisconnect(PIOCTLPARAMS aPIP);

#ifdef HOOK_DEBUG               /* This is for debugging hook logic. */

enum HookProgress {
  HookProgressGotPacket = 1,
  HookProgressGotBuffer,
  HookProgressGotBroadcast,
  HookProgressGotRequest
};

#define HOOK_PROGRESS(milestone) \
if (billgpsContext.hookProgress < HookProgress##milestone) { \
  billgpsContext.hookProgress = HookProgress##milestone; \
}


#else  /* HOOK_DEBUG not defined */

#define HOOK_PROGRESS(milestone)

#endif

/*
   PROTOCOL CONTEXT

   We only have one of these structures for the Protocol VxD.
   When updating this structure, remember that there's a reason for the
   way its members are ordered: the needs of the IOCTL to dump the driver's
   guts.  Make sure the global stuff comes first, followed by the adapter
   contexts, and lastly (what doesn't need to be dumped) the packet buffer.
*/

enum { BgpsNAdapters = 8, NHookBufferEntries = 8 };
typedef struct _ProtocolBlock {
  NDIS_HANDLE ndisProtocolHandle;     // returned from NdisRegisterProtocol
  NDIS_STATUS registerStatus;
  NDIS_PROTOCOL_CHARACTERISTICS protocolCharacteristics;
  LIST_ENTRY OpenList;
#ifdef HOOK_DEBUG
  enum HookProgress hookProgress;
  UINT bufferLengths[NHookBufferEntries];
#endif
#if 0                           // moved or obsolete
  OVERLAPPED* ovrlp;
  NDIS_HANDLE packetPoolHandle;
  NDIS_HANDLE bufferPoolHandle;
  PNDIS_PACKET packetP;
  PNDIS_BUFFER bufferP;
  UINT packetFilter;            // NdisRequest buffer; can't be on stack!
  UINT bindingHighWater;
  AdapterContext adapterContexts[BgpsNAdapters];
  /* Make this last; we won't dump it */
  BillgpBuf billgpBuf;
#endif
} ProtocolBlock, *PProtocolBlock;

extern ProtocolBlock billgpsContext;

#endif // __BILLGPS_H__


